function [res, paramsH] = trainH(paramsGen, paramsH)
%simulates conditioning according to Justin Harris' model WITHOUT taking
%into account associations between CS elements.

% The generel parameters passed by paramsGen are
% Stimulusname: string name of all CS
% Testname:     "
% Kont:         US paired with CS
% KontTest:     "
% beta:         beta of USs
% AnzVp:        number of subjects
% CS:           CS-sequence: noTrial*AnzVp-matrix
% US:           US-sequence: noTrial*AnzVp-matrix
% code:         stimuluscode of all CS: a line defines the occurrence of each single stimulus in a CS (without unique cue or similar)
% testcode:     "
% block:        CS that occur during training , specified by their line number in Stimulusname

%The parameters received through paramsH are :
% runs          The number of runs to perform
% noTrials      The number of trials
% noElems       The number of elements used to represent each single CS
% density       The connection density in percent/100
% gain          The buffer gain, i.e., the factor by which the activation
%               of elements in the buffer is multiplied.
% beta          The Rescorla-Wagner learning rule beta
% cap           The capacity of the attention buffer
% fraction      The overlap of CS elements in percent/100
% simple        Indicates whether to use the simplified learning equation or
%               not.
% alphas        The alphas of single CS
% alphadefault:     default value defined in default.conf
% trials    CS (compounds) presented on each trial
% eval         The CS (compounds) to be evaluated

%The return value of the function (res) is the CR for each of the CS
%(compounds) specified in eval.


% Copyright (C) 2006-2008  Anna Thorwart and Holger Schultheis


trials=[paramsGen.code abs(paramsGen.Kont)];
eval=trials;
noTrials=size(paramsGen.CS,1);
assignin('base', 'evalCode', eval);

power = 1; %power is a variable describing the exponent to which
% input strengths are raised when transforming inputs to activation
% weights.  When power = 1, the relationship between input and activation
% weight is linear.  See footnote #3 (p592) of Harris (2006) Psych Rev,
% 113.

noCS = size(trials,2) - 1; %number of single CS
simulatedCR=zeros(size(eval,1),sum(noTrials)); %matrix into which final simulation data will be placed
simulatedAmatrix=zeros(size(eval,1),sum(noTrials));
total = giveTotal(paramsH.noElems,paramsH.fraction,noCS); %number of elements used to actually represent each CS (compound)

%check alphas
NoAlphas_be=size(paramsH.alphas,2); %number of alphas needed

if noCS ~=NoAlphas_be
    meldung=['The number of alphas doesn''t fit. There should be ' num2str(noCS) ', but there are ' num2str(NoAlphas_be) '. Do you wish to run the simulation using the default value for all alphas?.'];
    answer= questdlg(meldung, 'Miscount!', 'Yes', 'Cancel', 'Yes');
    if strcmp('Yes', answer);
        paramsH.alphas=repmat(paramsH.alphadefault,1,noCS);
    else
        res=[];
        return
    end
end


for i=1:paramsH.runs
    %setting up stimulus representations (and buffer capacity if necessary)
    
    %US %%TODO
    if paramsH.simple == 2
        L = paramsGen.US;
    else
        L_tmp=abs((randn(paramsH.noElems,1)+3)/3); %lambda (array of US elements with random weights, mean = 1) [need to take the absolute values to avoid negative US elements]
        for s=1:paramsGen.AnzVp
            for j=1:noTrials
                L(:,j,s)=paramsGen.US(j,s)* L_tmp;
            end
        end
    end
    noL=0; %lambda of non-reinforcement
    
    %     FIXME: in order to compare different discriminations, one has to use the same CSsRep
    %          Task1=load ('matlab.mat');
    %          L=Task1.USsRep;
%              assignin('base', 'USsRep', L);
    
    sparse=rand(paramsH.noElems,total) < paramsH.density;   %matrix of sparse CS-US connections
%     assignin('base', 'sparseA', sparse);
    
    %CSs
    CSsRep = setupCS(total,paramsH.fraction,paramsH.noElems,noCS, paramsH.alphas);
    %the previous line gives us what JH calls intensities. To determine
    %activations these intensities have to be raised to the power of the
    %corresponding variable (see below)
%     assignin('base', 'CSsRep', CSsRep);
    %     %FIXME: in order to compare different discriminations, one has to use the same CSsRep
%              Task1=load ('matlab.mat');
%              CSsRep=Task1.CSRep;
              
       
    %determine buffer capacity
    if paramsH.cap < 0
        [capacity, indx] = max(sum(CSsRep.^power,2));
        cutoff = min((CSsRep(indx,:)).^power);
        %this seems to be an arbitrary way of determining the cutoff. However,
        %as long as the way CS and US elements interact is not specified,
        %this cutoff should do. (Defined this way the cutoff will usually be
        %zero which means that any non-negative US element will be inside
        %the buffer.)
    else
        capacity = paramsH.cap;
        cutoff = getCutoff(CSsRep,power,capacity);
    end
    
    
    sumV = {}; %#ok<NASGU>
    CR=[];
    %     amatrix=zeros(noTrials, size(eval,1),paramsGen.AnzVp);
    %determine the cutoffs and element representations for the trials to
    %simulate
    trialReps = setupTrials(CSsRep,trials(:,1:(end-1)),capacity,paramsH.gain,power,sparse,paramsH.noElems);
    assignin('base', 'trialReps', trialReps);
    
    %same for the evaluation trials
    evalReps = setupTrials(CSsRep,eval(:,1:(end-1)),capacity,paramsH.gain,power,sparse,paramsH.noElems);
    assignin('base', 'evalReps', evalReps);
    
    for s=1:paramsGen.AnzVp
        
        Vs = zeros(paramsH.noElems,total);
        %start simulating the trials
        for j=1:noTrials
            %registering conditioned responses
            for k=1:size(eval,1)
                CR(k,j,s) = sum(sum(Vs.*evalReps{k},2));
            end
            %computing activation of US elements
            sumV = sum(Vs.*trialReps{paramsGen.CS(j,s)},2);
            
            %overall US activation in trial
            OutputVP(j)=sum(sumV);
            
            %updating beta
            beta = paramsGen.beta(paramsGen.CS(j,s))*.1; %NOTE: beta values are scaled down to prevent response from building up
            
            %updating association strengths
            %distinguish between simplified learning (using the standard delta
            %rule) and more complex learning (using buffer threshold during
            %learning).
            if paramsH.simple == 2
                if paramsGen.US(j,s)> 0 %%US present %TODO: wie gehen wir mit negativen lambda im Harris Modell um?
                    Vs = Vs + (repmat((paramsH.gain - 1)*beta*(L(j,s)-sumV), 1, total).*trialReps{paramsGen.CS(j,s)});
                else %US not present
                    Vs = Vs + (repmat( beta*(noL-sumV),1,total).*trialReps{paramsGen.CS(j,s)});
                end
            else
                %as long as there are no clear conceptions about the competition of
                %CS and US elements for entry to the buffer, the threshold for the
                %learning equations is not unambiguously specified (could be either
                %the same or different thresholds for different CS compounds).
                %Since JH uses the same threshold for all CS, we do so too.
                
                if paramsGen.US(j,s) > 0 %US present
                    controll=(L(:,j,s)-sumV)>=cutoff; %#ok<NASGU>
                    Vs = Vs + (repmat (paramsH.gain*beta*(L(:,j,s)-sumV) .* ((L(:,j,s)-sumV)>=cutoff), 1, total) .*trialReps{paramsGen.CS(j,s)}) ...
                        + ( repmat( -1*beta*abs(L(:,j,s)-sumV).*( (L(:,j,s)-sumV)<cutoff) ,1,total) .*trialReps{paramsGen.CS(j,s)});
                else %US NOT present
                    Vs = Vs + (repmat( beta*(noL-sumV),1,total).*trialReps{paramsGen.CS(j,s)});
                end
                
            end
            
        end
        assignin('base', 'endVs', Vs);
        amatrix(:,:,s)=sorttrials(OutputVP, paramsGen.CS(:,s), length(paramsGen.Stimulusname)); %#ok<*AGROW>
        assignin('base', 'amatrix', amatrix);
    end
    CR(:,:, s+1)=mean(CR,3); %average across subjects
    simulatedCR=simulatedCR+CR(:,:,s+1); %combines data across simulation runs
    amatrix(:,:,s+1)=mean(amatrix,3);
    simulatedAmatrix=simulatedAmatrix+amatrix(:,:,s+1)';
end

amat=simulatedAmatrix/paramsH.runs;
res = simulatedCR/paramsH.runs; %averages data across simulation runs
assignin('base', 'meanCR', res)



% plotting

trainingstring=[];
for ii=1:length(paramsGen.block)
    trainingstring = [trainingstring, strjoin(paramsGen.Stimulusname(paramsGen.block{ii})',  ','),'|'];
end

% create string with parameters
if size(paramsH.alphas,2)<7 && length(beta) < 7
    titlestring ={['connection density=', num2str(paramsH.density), ', common elements=', num2str(paramsH.fraction), ', gain=', num2str(paramsH.gain), ';'];...
        ['\alpha=', mat2str(paramsH.alphas'),'; \beta=', mat2str(paramsGen.beta*.1), '; phases: ', trainingstring]};
elseif size(paramsH.alphas,2)>7 && length(paramsGen.beta) <= 7
    titlestring= {['connection density=', num2str(paramsH.density), ', common elements=', num2str(paramsH.fraction), ', gain=', num2str(paramsH.gain),  ';'];...
        ['\alpha= xxx; \beta=', mat2str(paramsGen.beta*.1),'; phases: ', trainingstring]};
elseif size(paramsH.alphas,2)<=7 && length(paramsGen.beta) > 7
    titlestring= {['connection density=', num2str(paramsH.density), ', common elements=', num2str(paramsH.fraction), ', gain=', num2str(paramsH.gain),  ';'];...
        ['\alpha=', mat2str(paramsH.alphas'),'; \beta=xxx ; phases: ', trainingstring]};
else
    titlestring= {['connection density=', num2str(paramsH.density), ', common elements=', num2str(paramsH.fraction), ', gain=', num2str(paramsH.gain),  ';'];...
        ['\alpha=xxx; \beta=xxx ; phases: ', trainingstring]};
end


if paramsH.activationPlots
    plotOutputActivation(res','activation by teststimuli: Harris ', titlestring,paramsGen.Stimulusname );
%     plotOutputActivation(amat','activation by presented stimulus: Harris', titlestring, paramsGen.Stimulusname);
end


%save the output activation matrix
speicherpath=paramsH.savePath;
if ~isempty(speicherpath{1});
    dlmwrite ([speicherpath{1},'\', speicherpath{2}, '_testactivation.txt'], res, '\t') ;
    dlmwrite ([speicherpath{1},'\', speicherpath{2}, '_ouputactivation.txt'], amat, '\t') ;

end






%%%%%%%%%%%%%%%%% helper functions %%%%%%%%%%%%%%%%%

function total = giveTotal(noElems,fraction,noCS)
if fraction > 0
    total = round(noElems/fraction);
    %and simulation without common elements
else
    total = noElems * noCS;
end

function csrep = setupCS(total,fraction,noElems,noCS, salience)
csrep = [];
if fraction > 0
    for i=1:noCS
        tmpCS = salience(i)*[(rand(1,total) < fraction).*((randn(1,total)+3)/6)]; %#ok<NBRAK>
        tmpCS = tmpCS.*(tmpCS > 0);
        csrep = [csrep;tmpCS];
    end
else
    for i=1:noCS
        tmpCS = zeros(1,total);
        tmpCS(1+noElems*(i-1):(noElems*i)) = salience(i)*[(randn(1,noElems)+3)/6]; %#ok<NBRAK>
        tmpCS = tmpCS.*(tmpCS > 0);
        csrep = [csrep;tmpCS];
    end
end

function reps = setupTrials(CSsRep,trials,capacity,gain,power,sparse,noElems)
reps = {};

for i=1:size(trials,1)
    tmp = sum(CSsRep((trials(i,:)>0),:),1).^power;
    tmpDesc = fliplr(sort(tmp)); %#ok<FLPST>
    cutoff = max((cumsum(tmpDesc) > capacity).*tmpDesc);
    interRep = (gain*((tmp > cutoff).*tmp)) + ((tmp <= cutoff).*tmp);
    reps = [reps,{(sparse.*repmat(interRep,noElems,1))}];
end


function c = getCutoff(reps,power,capacity)
cuts = [];
for i=1:size(reps, 1)
    tmp = sort(reps(i, reps(i,:) > 0), 'descend');
    cuts = [cuts, max((cumsum(tmp) > capacity).*tmp)];
end
c = mean(cuts);



%--------------------------------------------------------------------------
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.